package jp.co.powerbeans.common.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

public final class PBControlUtil {

  private PBControlUtil() {
    super();
  }

  // public static Table createSimpleTable(Composite composit, String[] cols, int[] wids) {
  // return createSimpleTable(composit, cols, wids, 1);
  // }
  //
  public static Table createSimpleTable(Composite composit, String[] cols, int[] wids, int colspan) {
    return createSimpleTable(composit, cols, wids, colspan, 0);
  }
  
  public static Table createSimpleTable(Composite composit, String[] cols, int[] wids, int colspan, int height) {
    ChkConvSwt.assertEquals(cols.length, wids.length);
    // テーブルの作成
    Table table = new Table(composit, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
    GridData tableGrid = new GridData(GridData.FILL_BOTH);
    tableGrid.horizontalSpan = colspan;
    tableGrid.horizontalAlignment = GridData.FILL;
    // tableGrid.verticalAlignment = GridData.FILL;
    tableGrid.grabExcessHorizontalSpace = true;
    // tableGrid.grabExcessVerticalSpace = true;
    if (height > 0) {
      tableGrid.minimumHeight = height;
    }
    table.setLayoutData(tableGrid);
    // 線を表示する
    table.setLinesVisible(true);
    // ヘッダを可視にする
    table.setHeaderVisible(true);
    // 列のヘッダの設定
    for (int i = 0; i < cols.length; i++) {
      TableColumn col = new TableColumn(table, SWT.LEFT);
      col.setText(cols[i]);
      col.setWidth(wids[i]);
    }

    return table;
  }

  public static TableTree createTreeTable(Composite composit, String[] cols, int[] wids, int colspan) {
    ChkConvSwt.assertEquals(cols.length, wids.length);
    // テーブルの作成
    TableTree table = new TableTree(composit, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
    GridData tableGrid = new GridData();
    tableGrid.horizontalSpan = colspan;
    tableGrid.horizontalAlignment = GridData.FILL;
    tableGrid.verticalAlignment = GridData.FILL;
    tableGrid.grabExcessHorizontalSpace = true;
    tableGrid.grabExcessVerticalSpace = true;
    tableGrid.minimumHeight = 200;
    table.setLayoutData(tableGrid);
    // 線を表示する
    table.getTable().setLinesVisible(true);
    // ヘッダを可視にする
    table.getTable().setHeaderVisible(true);
    // 列のヘッダの設定
    for (int i = 0; i < cols.length; i++) {
      TableColumn col = new TableColumn(table.getTable(), SWT.LEFT);
      col.setText(cols[i]);
      col.setWidth(wids[i]);
    }

    return table;
  }

  public static Button createSimpleButton(PBComposite composit, String label, String cmd) {
    return createSimpleButton(composit, label, cmd, null);
  }

  public static Button createSimpleButton(PBComposite composit, String label, String cmd,
      Object[] args) {
    GridData gridData = new GridData();
    Button button = new Button(composit, SWT.NONE);
    button.setText(label);
    button.setLayoutData(gridData);
    gridData.horizontalAlignment = SWT.FILL;
    gridData.grabExcessHorizontalSpace = true;
    button.addSelectionListener(new CmdSelectionAdapter(composit, cmd, args));

    return button;
  }

  public static Button createCheckBox(Composite composit, String label) {
    GridData gridData = new GridData();
    Button button = new Button(composit, SWT.CHECK);
    button.setText(label);
    button.setLayoutData(gridData);
    // gridData.horizontalAlignment = SWT.FILL;
    // gridData.grabExcessHorizontalSpace = true;

    return button;
  }

  public static void addRow(Table table, String[] data) {
    TableItem item = new TableItem(table, SWT.NULL);
    item.setText(data);
  }

  public static void insertRowTop(Table table, String[] data) {
    insertRow(table, data, 0);
  }

  public static void insertRow(Table table, String[] data, int index) {
    TableItem item = new TableItem(table, SWT.NULL, index);
    item.setText(data);
  }

  public static Label createLabel_forceCenter(Composite composit, String label) {
    org.eclipse.swt.widgets.Label l = new org.eclipse.swt.widgets.Label(composit, SWT.CENTER
        | SWT.CENTER);
    l.setText(label);

    GridData gridData = new GridData();
    gridData.horizontalAlignment = SWT.CENTER;
    gridData.verticalAlignment = SWT.CENTER;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    // gridData.widthHint = 200;
    l.setLayoutData(gridData);
    return l;
  }
  public static Label createLabel_forceCenter_FILL(Composite composit, String label) {
    org.eclipse.swt.widgets.Label l = new org.eclipse.swt.widgets.Label(composit, SWT.CENTER
        | SWT.CENTER);
    l.setText(label);

    GridData gridData = new GridData();
    gridData.horizontalAlignment = SWT.FILL;
    gridData.verticalAlignment = SWT.BOTTOM;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.heightHint = composit.getSize().y / 2;
    l.setLayoutData(gridData);
    return l;
  }

  public static Label createLabel(Composite composit, String label, int hstyle, int vstyle,
      int colspan) {
    org.eclipse.swt.widgets.Label l = new org.eclipse.swt.widgets.Label(composit, hstyle | vstyle);
    l.setText(label);

    if (hstyle != 0 || vstyle != 0) {
      GridData gridData = new GridData();
      gridData.horizontalAlignment = hstyle;
      gridData.verticalAlignment = vstyle;
      gridData.horizontalSpan = colspan;
      // gridData.grabExcessHorizontalSpace = true;
      // gridData.grabExcessVerticalSpace = true;
      // gridData.widthHint = 200;
      l.setLayoutData(gridData);
    }
    return l;
  }

  public static Label createLabel(Composite composit, String label, int hstyle, int vstyle) {
    return createLabel(composit, label, hstyle, vstyle, 1);
  }

  public static Label createLabel(Composite composit, String label, int hstyle) {
    return createLabel(composit, label, hstyle, SWT.BEGINNING);
  }

  public static Label createLabel_hvcenter(Composite composit, String label) {
    return createLabel(composit, label, SWT.CENTER, SWT.CENTER);
  }

  public static Label createLabel_left(Composite composit, String label) {
    return createLabel(composit, label, SWT.LEFT);
  }

  public static Label createLabel_right(Composite composit, String label) {
    return createLabel(composit, label, SWT.RIGHT);
  }

  public static void addSeparator(Menu menu) {
    new MenuItem(menu, SWT.SEPARATOR);
  }

  /**
   * 部品を複数(たて1列)配置できるCompositeを生成
   * 
   * @param 親shell
   * @return composite
   */
  public static Composite createNRow1ColComposit(Shell shell) {
    Composite comp = new Composite(shell, SWT.NO_FOCUS);
    comp.setLayout(new GridLayout(1, true));
    GridData compGrid = new GridData();
    compGrid.horizontalAlignment = GridData.FILL;
    compGrid.verticalAlignment = GridData.FILL;
    compGrid.grabExcessHorizontalSpace = true;
    compGrid.grabExcessVerticalSpace = true;
    comp.setLayoutData(compGrid);
    return comp;
  }

  /**
   * 部品を複数(たて2列)配置できるCompositeを生成
   * 
   * @param 親shell
   * @return composite
   */
  public static Composite createNRow2ColComposit(Shell shell) {
    Composite comp = new Composite(shell, SWT.NO_FOCUS);
    comp.setLayout(new GridLayout(2, true));
    GridData compGrid = new GridData();
    compGrid.horizontalAlignment = GridData.FILL;
    compGrid.verticalAlignment = GridData.FILL;
    compGrid.grabExcessHorizontalSpace = true;
    compGrid.grabExcessVerticalSpace = true;
    comp.setLayoutData(compGrid);
    return comp;
  }

  /**
   * 部品を複数(たて3列)配置できるCompositeを生成
   * 
   * @param 親shell
   * @return composite
   */
  public static Composite createNRow3ColComposit(Shell shell) {
    Composite comp = new Composite(shell, SWT.NO_FOCUS);
    comp.setLayout(new GridLayout(3, true));
    GridData compGrid = new GridData();
    compGrid.horizontalAlignment = GridData.FILL;
    compGrid.verticalAlignment = GridData.FILL;
    compGrid.grabExcessHorizontalSpace = true;
    compGrid.grabExcessVerticalSpace = true;
    comp.setLayoutData(compGrid);
    return comp;
  }

  // /**
  // * shellにテキストボックスを追加
  // * @param shell
  // * @return テキストボックス
  // */
  // public static Text addTextBox(Shell shell) {
  //    
  // return addTextBox(shell, SWT.SINGLE, 1);
  // Text text = new Text(shell, SWT.SINGLE | SWT.BORDER);
  // GridData gridData = new GridData();
  // gridData.horizontalAlignment = GridData.FILL;
  // gridData.verticalAlignment = SWT.BEGINNING;
  // gridData.grabExcessHorizontalSpace = true;
  // text.setLayoutData(gridData);
  //
  // return text;
  // }

  /**
   * shellにテキストボックスを追加
   * 
   * @param shell
   * @param height
   * @return テキストボックス
   */
  public static Text addTextBox(Composite composite, int style, int colspan, int height) {
    Text text = new Text(composite, SWT.BORDER | style);
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.verticalAlignment = GridData.BEGINNING;
    gridData.horizontalSpan = colspan;
    gridData.grabExcessHorizontalSpace = true;
//    gridData.grabExcessVerticalSpace = true;
    if (height > 0) {
      gridData.heightHint = height;
    }
    text.setLayoutData(gridData);

    return text;
  }

  /**
   * 文字列幅のボタンを追加
   * 
   * @param comp
   * @param label
   *          ラベル
   * @return Button
   */
  public static Button addSimpleButton(Composite comp, String label) {

    GridData gridData = new GridData();
    Button button = new Button(comp, SWT.NONE);
    button.setText(label);
    button.setLayoutData(gridData);
    gridData.horizontalAlignment = SWT.FILL;
    gridData.verticalAlignment = SWT.BEGINNING;
    gridData.grabExcessHorizontalSpace = true;

    return button;
  }

  /**
   * ラジオボタンを追加
   * 
   * @param comp
   * @param labels
   * @param vals
   * @return
   */
  public static Button[] addRadioButtons(Composite comp, String[] labels, int[] vals) {
    Button[] bnts = new Button[labels.length];

    for (int i = 0; i < bnts.length; i++) {
      GridData gridData = new GridData();
      Button button = new Button(comp, SWT.RADIO);
      button.setText(labels[i]);
      button.setLayoutData(gridData);
      gridData.horizontalAlignment = SWT.FILL;
      gridData.grabExcessHorizontalSpace = true;
      button.setData("" + vals[i]);
      bnts[i] = button;
    }
    return bnts;
  }

  /**
   * @param comp
   * @param style
   * @return
   */
  public static Combo createComboBox(Composite comp, int style, int width) {
    return createComboBox(comp, style, 1, width);
    // // GridData gridData = new GridData();
    // Combo combo = new Combo(shell, SWT.BEGINNING | SWT.TOP | style);
    //
    // if (width > 0) {
    // GridData gridData = new GridData();
    // gridData.widthHint = width;
    // combo.setLayoutData(gridData);
    // }
    // // gridData.heightHint = 30;
    // // combo.setLayoutData(gridData);
    // // gridData.horizontalAlignment = SWT.FILL;
    // // gridData.verticalAlignment = SWT.BEGINNING;
    // // gridData.grabExcessHorizontalSpace = true;
    // // gridData.grabExcessVerticalSpace = true;
    // return combo;
  }

  /**
   * @param shell
   * @param style
   * @param width
   * @param width2
   * @return
   */
  public static Combo createComboBox(Composite composit, int style, int colspan, int width) {
    Combo combo = new Combo(composit, SWT.BEGINNING | SWT.TOP | style);

    GridData gridData = new GridData();
    if (width > 0) {
      gridData.widthHint = width;
    }
    // gridData.grabExcessHorizontalSpace = true;
    gridData.horizontalSpan = colspan;
    gridData.verticalAlignment = SWT.BEGINNING;
    gridData.horizontalAlignment = SWT.BEGINNING;
    combo.setLayoutData(gridData);
    return combo;
  }

  /**
   * グリッドレイアウトを指定
   * 
   * @param composit
   *          指定するShell
   * @param col
   *          列数
   */
  public static void setGridLayout(Composite composit, int col) {
        GridLayout layout = new GridLayout(col, true);
        layout.numColumns = col;
        layout.makeColumnsEqualWidth = false;
        composit.setLayout(layout);
  }

  /**
   * グリッドレイアウトを指定
   * 
   * @param composit
   *          指定するShell
   * @param col
   *          列数
   */
  public static void setGridLayout_FILL(Composite composit, int col) {
        GridLayout layout = new GridLayout(col, true);
        layout.numColumns = col;
        layout.makeColumnsEqualWidth = false;
        composit.setLayout(layout);
  }

	/**
	 * Groupを生成
	 * @param composit 親 
	 * @param label ラベル
	 * @param col 列スパン数
	 * @return Group
	 */
	public static Group createGroupChadowEthedOut(Composite composit, String label, int col, int horAlign) {
  return createGroupChadowEthedOut(composit, label, col, horAlign, true);
	}

/**
 * Groupを生成
 * @param composit 親 
 * @param label ラベル
 * @param col 列スパン数
 * @param equal_width 列幅を同じにするかどうか
 * @return Group
 */
public static Group createGroupChadowEthedOut(Composite composit, String label, int col, int horAlign, boolean equal_width) {
  Group g = new Group(composit, SWT.SHADOW_ETCHED_OUT  | SWT.EMBEDDED);
  g.setText(label);
  GridData gridData = new GridData();
  gridData.horizontalAlignment = horAlign;
  gridData.grabExcessHorizontalSpace = true;
  g.setLayoutData(gridData);
  
  GridLayout layout = new GridLayout(col, true);
  layout.makeColumnsEqualWidth = equal_width;
  g.setLayout(layout);
  
  return g;
}

	/**
	 * Groupを生成
	 * @param composit 親 
	 * @param label ラベル
	 * @param col 列スパン数
	 * @return Group
	 */
	public static Group createGroupChadowEthedOut_BEGIN(Composite composit, String label, int col, boolean equal_width) {
		return createGroupChadowEthedOut(composit, label, col, SWT.FILL, equal_width);
	}

	/**
	 * Groupを生成
	 * @param composit 親 
	 * @param label ラベル
	 * @param col 列スパン数
	 * @return Group
	 */
	public static Group createGroupChadowEthedOut_FILL(Composite composit, String label, int col, boolean equal_width) {
		return createGroupChadowEthedOut(composit, label, col, SWT.FILL, equal_width);
	}

  /**
   * Browserを生成
   * @param comp 親
   * @param colspan 
   * @return ブラウザ
   */
  public static Browser createBrowser(Composite comp, int colspan) {
    Browser browser = new Browser(comp, SWT.BORDER);
//    
//    Group g = new Group(comp, SWT.SHADOW_ETCHED_OUT  | SWT.EMBEDDED);
    GridData gridData = new GridData();
    gridData.horizontalAlignment = SWT.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.heightHint = 300;
    browser.setLayoutData(gridData);
    
    setSpan(browser, colspan, 1);
    
    return browser;
  }

  /**
   * @param c
   * @param colspan
   * @param rowspan
   */
  public static void setSpan(Control c, int colspan, int rowspan) {
    GridData grid = (GridData) c.getLayoutData();
    grid.horizontalSpan = colspan;
    grid.verticalSpan = rowspan;
  }

  /**
   * @param button
   */
  public static void setStyleBottom(Control c) {
    GridData grid = (GridData) c.getLayoutData();
    grid.verticalAlignment = SWT.BOTTOM;
    grid.grabExcessVerticalSpace = true;
  }
	

}
