package jp.co.powerbeans.common.swt;

import java.awt.DisplayMode;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import jp.co.powerbeans.common.db.DBManager;
import jp.co.powerbeans.common.dict.DictUtil;
import jp.co.powerbeans.common.log.Log;
import jp.co.powerbeans.docbuild.PBDocmentBuilderFactory;
import jp.co.powerbeans.docbuild.PBDocumentBuilder;

import org.apache.commons.beanutils.BeanUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

public abstract class PBSwt implements PBCmdDispatch, DisposeListener {

  private static final String CSS = "pbdoc.css";

  private static final String TEMPLATE_DIR = "template/";

  private static final String CHARSET = "Shift_JIS";

  private static final int DEF_WIDTH;

  private static final int DEF_HEIGHT;

  private static final double PER_WINDOW = 0.8;
  static {
    GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
    GraphicsDevice device = ge.getDefaultScreenDevice();

    DisplayMode originalDisplayMode = device.getDisplayMode();
    // DisplayMode[] modes = device.getDisplayModes();
    DEF_WIDTH = (int) (originalDisplayMode.getWidth() * PER_WINDOW);
    DEF_HEIGHT = (int) (originalDisplayMode.getHeight() * PER_WINDOW);
    // int width = 800;
    // int height = 800;
    // height = (int) (modes[device.getType()].getHeight() * 0.8);
    // width = (int) (modes[device.getType()].getWidth() * 0.8);
    // DEF_WIDTH = width;
    // DEF_HEIGHT = height;
  }

  /**
   * メインShell
   */
  protected org.eclipse.swt.widgets.Shell sShell;

  /**
   * コントロール名-コントロール関連付けMap
   */
  private Map controlMap;

  protected String resultCmd;

  private Image img;

  protected PBSwt() {
    super();

    this.sShell = null;
    this.controlMap = new HashMap();
    this.resultCmd = "";
  }

  /**
   * Shellを生成
   * 
   * @param title
   * @param parent_shell
   */
  protected void createSShell(String title) {
    createSShell(title, DEF_WIDTH, DEF_HEIGHT, false);
  }

  /**
   * Shellを生成
   * 
   * @param title
   * @param parent_shell
   */
  protected void createSShell(String title, boolean modal) {
    createSShell(title, DEF_WIDTH, DEF_HEIGHT, modal);
  }

  /**
   * Shellを生成
   * 
   * @param title
   * @param width
   * @param height
   * @param parent_shell
   */
  protected void createSShell(String title, int width, int height) {
    createSShell(title, width, height, false);
  }

  /**
   * Shellを生成
   * 
   * @param title
   * @param width
   * @param height
   * @param parent
   */
  protected void createSShell(String title, int width, int height, boolean modal) {
    if (modal && Display.getCurrent() != null) {
      sShell = new org.eclipse.swt.widgets.Shell(Display.getCurrent().getActiveShell(), SWT.CLOSE
          | SWT.PRIMARY_MODAL);
    } else {
      sShell = new org.eclipse.swt.widgets.Shell();
    }

    sShell.setText(title);
    sShell.setSize(new org.eclipse.swt.graphics.Point(width, height));

    setGridLayout(1);

    // set image
    ImageData imgData = new ImageData("image/icon.gif");
    img = new Image(sShell.getDisplay(), imgData); // ① Imageオブジェクトを生成
    sShell.setImage(img);
    this.sShell.addDisposeListener(this);
  }

  public void widgetDisposed(DisposeEvent e) {
    this.img.dispose();
	if (this.sShell.getShell().isDisposed()) {
		this.sShell.getShell().dispose();
	}
  }

  /**
   * sShellにグリッドレイアウトを設定(一般的なレイアウト)
   * 
   * @param col
   *          列数
   */
  protected void setGridLayout(int col) {
    PBControlUtil.setGridLayout(this.sShell, col);
  }
  /**
   * sShellにグリッドレイアウトを設定(一般的なレイアウト)
   * 
   * @param col
   *          列数
   */
  protected void setGridLayout_FILL(int col) {
    PBControlUtil.setGridLayout_FILL(this.sShell, col);
  }

  /**
   * sShellを表示(モーダル表示)
   * 
   * @return 処理コマンド
   */
  public String show() {
    return show(true);
  }

  /**
   * sShellを表示
   * 
   * @param modal
   *          true モーダル表示(メッセージDispatch有り), false 非モーダル表示(メッセージDispatch無し)
   * @return 処理コマンド
   */
  public String show(boolean modal) {
    org.eclipse.swt.widgets.Display display = org.eclipse.swt.widgets.Display.getDefault();
    this.sShell.open();

    if (modal) {
      while (!this.sShell.isDisposed()) {
        if (!display.isDisposed() && !display.readAndDispatch()) {
          display.sleep();
        }
      }
    }
    // アプリは終了しない
    // display.dispose();
    // 処理結果を返す
    return this.resultCmd;
  }

  /**
   * コマンド対応メソッドを実行
   * 
   * @param cmd
   *          コマンド
   * @return true 実行, false 未実行
   */
  public boolean dispatchCmd(String cmd, Object[] args) {
    if (cmd == null || cmd.length() <= 1) {
      return false;
    }

//  砂時計
    this.sShell.setCursor(new Cursor(null, SWT.CURSOR_WAIT));
    
    String subname = cmd.substring(0, 1).toUpperCase() + cmd.substring(1);
    try {
      // 実行メソッドの引数を作成
      Class[] types = null;
      if (args != null) {
        types = new Class[args.length];
        for (int i = 0; i < args.length; i++) {
          types[i] = args[i].getClass();
        }
      } else if (cmd.indexOf(",") != -1) {
        // コマンド名に "," が存在する場合は区切ってStringで渡す
        String[] cmd_part = cmd.split(",");
        subname = cmd_part[0].substring(0, 1).toUpperCase() + cmd_part[0].substring(1);
        types = new Class[cmd_part.length - 1];
        for (int i = 0; i < types.length; i++) {
          types[i] = String.class;
        }
        // 値
        args = new String[types.length];
        for (int i = 0; i < args.length; i++) {
          args[i] = cmd_part[i + 1];
        }
      }
      
      Method method = this.getClass().getMethod("cmd" + subname, types);
      method.invoke(this, args);
      return true;

    } catch (Exception e) {
      e.printStackTrace();
    } finally {
//    	 元に戻す
    	setCursorArror();
    }
    return false;
  }

  /**
	 * 
	 */
	protected void setCursorWait() {
		setCursor(SWT.CURSOR_WAIT);
	}

	/**
	 * 
	 */
	protected void setCursorArror() {
		setCursor(SWT.CURSOR_ARROW);
	}

	/**
	 * @param cursor_arrow
	 */
	protected void setCursor(int cursor_arrow) {
		if (this.sShell != null && !this.sShell.isDisposed()) {
			this.sShell.setCursor(new Cursor(null, cursor_arrow));
		}
	}

/**
   * 1階層メニューを作成して登録
   * 
   * @param labels
   *          表示ラベル
   * @param cmds
   *          実行コマンド(CMD.hoge)
   * @param menu_acc
   */
  protected void create1LayerMenu(String[] labels, String[] cmds) {
    create1LayerMenu(labels, cmds, null);
  }

  /**
   * 1階層メニューを作成して登録
   * 
   * @param labels
   *          表示ラベル
   * @param cmds
   *          実行コマンド(CMD.hoge)
   * @param menu_acc
   */
  protected void create1LayerMenu(String[] labels, String[] cmds, char[] menu_acc) {
    ChkConvSwt.assertEquals(labels.length, cmds.length);
    Menu menu = new Menu(this.sShell, SWT.BAR);
    this.sShell.setMenuBar(menu);

    for (int i = 0; i < labels.length; i++) {
      MenuItem menuItem0 = new MenuItem(menu, SWT.PUSH);
      menuItem0.addSelectionListener(new CmdSelectionAdapter(this, cmds[i]));
      if (menu_acc != null && i < menu_acc.length) {
        menuItem0.setAccelerator(SWT.CTRL | menu_acc[i]);
        menuItem0.setText(labels[i] + "  Ctrl+" + menu_acc[i]);
      } else {
        menuItem0.setText(labels[i]);
      }
    }
  }

  /**
   * 2階層メニューを作成して登録
   * 
   * @param labels1
   *          1階層目表示ラベル
   * @param labels2
   *          2階層目表示ラベル
   * @param cmds
   *          2階層目実行コマンド(CMD.hoge)
   * @param acc ショートカットキーアクセラレータ
   */
  protected void create2LayerMenu(String[] labels1, String[][] labels2, String[][] cmds) {
    create2LayerMenu(labels1, labels2, cmds, null);
  }
  
  /**
   * 2階層メニューを作成して登録
   * 
   * @param labels1
   *          1階層目表示ラベル
   * @param labels2
   *          2階層目表示ラベル
   * @param cmds
   *          2階層目実行コマンド(CMD.hoge)
   * @param acc
   *          ショートカットキーアクセラレータ
   */
  protected void create2LayerMenu(String[] labels1, String[][] labels2,
      String[][] cmds, int[][] acc) {
    ChkConvSwt.assertEquals(labels1.length, labels2.length);
    ChkConvSwt.assertEquals(labels2.length, cmds.length);

    Menu menu = new Menu(this.sShell, SWT.BAR);
    this.sShell.setMenuBar(menu);

    for (int i = 0; i < labels1.length; i++) {
      MenuItem menuItem0 = new MenuItem(menu, SWT.CASCADE);
      menuItem0.setText(labels1[i]);
      // menuItem0.addSelectionListener(new CmdSelectionAdapter(this, cmds[i]));
      Menu pmenu = new Menu(menuItem0);
      menuItem0.setMenu(pmenu);
      for (int j = 0; j < labels2[i].length; j++) {
        MenuItem subItem = new MenuItem(pmenu, SWT.PUSH);
        subItem.setText(labels2[i][j]);
        subItem.addSelectionListener(new CmdSelectionAdapter(this, cmds[i][j]));
        if (acc[i][j] > 0) {
          subItem.setAccelerator(acc[i][j]);
        }
      }
    }
  }

  /**
   * Tableダブルクリック時のコマンドを登録
   * 
   * @param table
   *          Table
   * @param cmd
   *          コマンド
   */
  protected void setCmdToTableOnDoubleClick(Table table, String cmd) {
    table.addMouseListener(new CmdDoubleClickMouseListener(this, cmd));
  }

  /**
   * Tableダブルクリック時のコマンドを登録
   * 
   * @param table
   *          Table
   * @param cmd
   *          コマンド
   */
  protected void setCmdToTableOnSingleClick(Table table, String cmd) {
    table.addMouseListener(new CmdSingleClickMouseListener(this, cmd));
  }
  
  /**
   * Table選択変更時のコマンドを登録
   * 
   * @param table
   *          Table
   * @param cmd
   *          コマンド
   */
  protected void setCmdToTableOnSelectChange(Table table, String cmd) {
    table.addSelectionListener(new CmdSelectionListener(this, cmd));
  }
  
  /**
   * ボタン(チェックボックス)変更時のコマンドを登録
   * @param button ボタン(通常チェックボックス)
   * @param cmd コマンド
   */
  protected void setCmdToButtonOnChange(Button button, String cmd) {
    button.addSelectionListener(new CmdSelectionListener(this, cmd));
  }
  
  /**
   * ショートカットキーに対応するコマンドを登録.
   * 実行メソッドは cmd + keyEvent.character
   * @param cmd
   */
  protected void setKeyListenerShortCut(String cmd) {
    this.sShell.addKeyListener(new ShortCutKeyListener(this, cmd));
  }

  /**
   * 部品を複数(たて1列)配置できるCompositeを生成
   * 
   * @return composite
   */
  protected Composite createNRow1ColComposit() {
    return PBControlUtil.createNRow1ColComposit(this.sShell);
  }

  /**
   * 部品を複数(たて2列)配置できるCompositeを生成
   * 
   * @return composite
   */
  protected Composite createNRow2ColComposit() {
    return PBControlUtil.createNRow2ColComposit(this.sShell);
  }

  /**
   * 部品を複数(たて2列)配置できるCompositeを生成
   * 
   * @return composite
   */
  protected Composite createNRow3ColComposit() {
    return PBControlUtil.createNRow3ColComposit(this.sShell);
  }

  /**
   * sShellにテキストボックスを追加
   * 
   * @param name
   *          項目名
   * @return テキストボックス
   */
  protected Text addTextBox(String name) {
    Text text = PBControlUtil.addTextBox(this.sShell, SWT.SINGLE, 1, 0);
    registControl(name, text);
    return text;
  }

  /**
   * sShellにテキストボックスを追加
   * 
   * @param name
   *          項目名
   * @return テキストボックス
   */
  protected Text addTextBox(Composite comp, String name) {
    Text text = PBControlUtil.addTextBox(comp, SWT.SINGLE, 1, 0);
    registControl(name, text);
    return text;
  }
  /**
   * sShellにテキストボックスを追加
   * 
   * @param name
   *          項目名
   * @param colspan
   *          列数
   * @return テキストボックス
   */
  protected Text addTextBoxReadOnly(String name, int colspan) {
    Text text = PBControlUtil.addTextBox(this.sShell, SWT.SINGLE | SWT.READ_ONLY, colspan, 0);
    registControl(name, text);
    return text;
  }

  /**
   * sShellにテキストボックスを追加
   * 
   * @param name
   *          項目名
   * @param colspan
   *          列数
   * @return テキストボックス
   */
  protected Text addTextBoxMultiReadOnly(String name, int colspan, int height) {
	return addTextBoxMultiReadOnly(this.sShell, name, colspan, height);
  }
  

  /**
   * compにテキストボックスを追加
   * 
   * @param name
   *          項目名
   * @param colspan
   *          列数
   * @return テキストボックス
   */
  protected Text addTextBoxMultiReadOnly(Composite comp, String name, int colspan, int height) {
    Text text = PBControlUtil.addTextBox(comp, SWT.MULTI | SWT.READ_ONLY, colspan, height);
    registControl(name, text);
    return text;
  }
  
  /**
   * sShellにテキストボックスを追加
   * 
   * @param name
   *          項目名
   * @param colspan
   *          列数
   * @return テキストボックス
   */
  protected Text addTextBoxMulti(String name, int colspan, int height) {
    Text text = PBControlUtil.addTextBox(this.sShell, SWT.MULTI, colspan, height);
    registControl(name, text);
    return text;
  }

  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel(String label) {
    return addLabel(label, SWT.RIGHT);
  }

  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel() {
    return addLabel("");
  }

  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel(String label, int hstyle) {
    return PBControlUtil.createLabel(this.sShell, label, hstyle);
  }

  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel_left(String label) {
    return addLabel(label, SWT.LEFT);
  }


  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel_left(Composite composite, String label) {
    return PBControlUtil.createLabel(composite, label, SWT.LEFT);
  }
  
  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel_right(String label) {
    return addLabel(label, SWT.RIGHT);
  }

  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel_right(Composite composite, String label) {
    return PBControlUtil.createLabel(composite, label, SWT.RIGHT);
  }
  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel_hvcenter(String label) {
    return PBControlUtil.createLabel_forceCenter(this.sShell, label);
  }
  /**
   * sShellにラベルを追加
   * 
   * @param label
   *          ラベル
   */
  protected Label addLabel_hvcenter_FILL(String label) {
    return PBControlUtil.createLabel_forceCenter_FILL(this.sShell, label);
  }

  /**
   * @param label
   * @param colspan
   */
  protected void addLabelWithColspan(String label, int colspan) {
    PBControlUtil.createLabel(this.sShell, label, SWT.LEFT, SWT.BEGINNING, colspan);
  }

  /**
   * @param comp
   * @param label
   */
  protected void addLabel(Composite comp, String label) {
    PBControlUtil.createLabel(comp, label, SWT.BEGINNING, SWT.BEGINNING);
  }

  /**
   * 文字列幅のボタンを追加
   * 
   * @param label
   *          ラベル
   * @param cmd
   *          実行コマンド
   * @return ボタン
   */
  protected Button addSimpleButton(String label, String cmd) {
	  return addSimpleButton(this.sShell, label, cmd);
  }
  /**
   * 文字列幅のボタンを追加
   * 
   * @param label
   *          ラベル
   * @param cmd
   *          実行コマンド
   * @return ボタン
   */
  protected Button addSimpleButton(Composite comp, String label, String cmd) {
    Button button = PBControlUtil.addSimpleButton(comp, label);
    button.addSelectionListener(new CmdSelectionAdapter(this, cmd));
    return button;
  }

  /**
   * 文字列幅のボタンを追加
   * 
   * @param label
   *          ラベル
   * @return ボタン
   */
  protected Button addSimpleButton(String label) {
    return addSimpleButton(label, "");
  }

  /**
   * 文字列幅,左寄りのボタンを追加
   * 
   * @param label
   *          ラベル
   * @param cmd
   *          コマンド
   * @return ボタン
   */
  protected Button addSimpleButton_Left(String label, String cmd) {
    return addSimpleButton(label, cmd);
  }

  /**
   * 文字列幅,中央寄りのボタンを追加
   * 
   * @param label
   *          ラベル
   * @param cmd
   *          コマンド
   * @return ボタン
   */
  protected Button addSimpleButton_Center(String label) {
    Button button = addSimpleButton(label);
    ((GridData) button.getLayoutData()).horizontalAlignment = GridData.CENTER;
    return button;
  }

  /**
   * OKボタンを追加. クリックすると cmdOk() メソッドが実行される
   * 
   * @return ボタン
   */
  protected Button addOKButton() {
    return addSimpleButton("OK(&O)", "ok");
  }

  /**
   * Cancelボタンを追加. クリックすると cmdCancel() メソッドが実行される.
   * 
   * @return ボタン
   */
  protected Button addCancelButton() {
    return addSimpleButton("Cancel(&C)", "cancel");
  }

  /**
   * OK,Cancelボタンを追加
   */
  protected void addOKCancelButton() {
    setStyleBottom(addOKButton());
    setStyleBottom(addCancelButton());
  }

  /**
   * @param ok_btn
   */
  private void setStyleBottom(Button button) {
    PBControlUtil.setStyleBottom(button);
  }

  /**
   * OKボタンの通常処理
   */
  public void cmdOk() {
    this.resultCmd = "ok";
    this.sShell.close();
  }

  /**
   * Cancelボタンの通常処理
   */
  public void cmdCancel() {
    this.resultCmd = "cancel";
    this.sShell.close();
  }

  /**
   * ラジオボタンを追加
   * 
   * @param name
   * @param labels
   * @param vals
   * @return
   */
  protected Button[] addRadioButtons(String name, String[] labels, int[] vals) {
    Button[] bnts = PBControlUtil.addRadioButtons(this.sShell, labels, vals);
    registControl(name, bnts);
    return bnts;
  }

  /**
   * JavaBeanの各プロパティ値をSWTコントロールに格納
   * 
   * @param obj
   *          コピー元Bean値
   */
  protected void copyModelToControl(Object obj) {
    if (obj == null) {
      return;
    }
    try {
      Class c = obj.getClass();
      BeanInfo info = Introspector.getBeanInfo(c);
      PropertyDescriptor[] properties = info.getPropertyDescriptors();

      for (int i = 0; i < properties.length; i++) {
        PropertyDescriptor p = properties[i];
        String name = p.getName();
        if ("class".equals(name) || p.getReadMethod() == null) {
          continue;
        }
        Method r_method = p.getReadMethod();
        Object val = r_method.invoke(obj, null);
        if (val == null) {
          continue;
        }
        String valstr = formatString(val);

        // 格納先Control取得
        Field to_field = getFieldSafe(name);
        if (to_field == null) {
          continue;
        }

        ControlData cd = ControlDataFactory.createInstance(getControl(name));
        cd.setText(valstr);

      }
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (IntrospectionException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    }
  }

  /**
   * 引数のvalをデータ型毎に適した表示文字列を作成する
   * 
   * @param val
   * @return
   */
  private String formatString(Object val) {
    if (val instanceof Date) {
      // yyyy/mm/dd 型で取得
      return new SimpleDateFormat("yyyy/MM/dd").format(val);
    } else {
      return val.toString();
    }
  }

  private Field getFieldSafe(String name) {
    try {
      return this.getClass().getDeclaredField(name);
    } catch (NoSuchFieldException e) {
      // ありえる
    }
    return null;
  }

  protected Object copyControlToMode(Class c) {
    Object obj = null;
    try {
      BeanInfo info = Introspector.getBeanInfo(c);
      PropertyDescriptor[] properties = info.getPropertyDescriptors();
      obj = c.newInstance();

      for (int i = 0; i < properties.length; i++) {
        PropertyDescriptor p = properties[i];
        String name = p.getName();
        try {
          if ("class".equals(name) || p.getWriteMethod() == null) {
            continue;
          }

          // 値取得元Control取得
          Field to_field = getFieldSafe(name);
          if (to_field == null) {
            continue;
          }

          ControlData cd = ControlDataFactory.createInstance(getControl(name));
          String valstr = cd.getText();
          Object val = valstr;

          if (val == null) {
            continue;
          }

          // 値を格納
          if (p.getPropertyType() == Date.class) {
            val = ChkConvSwt.getDate(val.toString());
          }
          BeanUtils.setProperty(obj, name, val);
        } catch (ChkConvSwtRuntimeException e) {
          // エラー発生時のControlをフォーカス
          getControl(name)[0].setFocus();
          throw e;
        }
      }
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (IntrospectionException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    } catch (InstantiationException e) {
      e.printStackTrace();
    }
    return obj;
  }

  /**
   * コントロール名とコントロールをコントロールMapに登録する
   * 
   * @param name
   *          コントロール名(プロパティ名)
   * @param ctrl
   *          コントロール
   */
  private void registControl(String name, Control ctrl) {
    if (name != null && name.length() > 0) {
      this.controlMap.put(name, ctrl);
    }
  }

  /**
   * コントロール名とコントロールをコントロールMapに登録する
   * 
   * @param name
   *          コントロール名(プロパティ名)
   * @param ctrl
   *          コントロール
   */
  private void registControl(String name, Control[] ctrl) {
    if (name != null && name.length() > 0) {
      this.controlMap.put(name, ctrl);
    }
  }

  /**
   * コントロール名から生成済みコントロールを取得する.
   * 
   * @param name
   *          コントロール名
   * @return コントロール
   */
  private Control[] getControl(String name) {

    Object obj = this.controlMap.get(name);
    if (obj == null) {
      Log.warn("getControl(" + name + ") is null. ");
      return null;
    } else if (obj.getClass().isArray()) {
      // 配列の場合(ラジオボタンなど）選択値を値として返す
      return (Control[]) obj;
    } else {
      return new Control[] { (Control) obj };
    }
  }

  private Combo createReadOnlyComboBox() {
    return createComboBox(SWT.READ_ONLY, 1, 0);
  }

  private Combo createReadOnlyComboBox(Composite composite) {
    return createComboBox(composite, SWT.READ_ONLY, 1, 0);
  }

  private Combo createEditableComboBox(int colspan, int width) {
    return createComboBox(0, colspan, width);
  }

  private Combo createComboBox(int style, int colspan, int width) {
    return PBControlUtil.createComboBox(this.sShell, style, colspan, width);
  }

  private Combo createComboBox(Composite composite, int style, int colspan, int width) {
    return PBControlUtil.createComboBox(composite, style, colspan, width);
  }

  protected Combo addComboBoxByModel(Class model_cls, String col, String cmd) {
    return addComboBoxByModel((String)null, model_cls, col, cmd);
  }

  protected Combo addComboBoxByModel(Composite comp, Class model_cls, String col, String cmd) {
    return addComboBoxByModel(comp, null, model_cls, col, cmd);
  }

  protected Combo addComboBoxByModel(String name, Class model_cls, String col, String cmd) {
    return addComboBoxByModel(this.sShell, name, model_cls, col, cmd);
  }
  protected Combo addComboBoxByModel(Composite comp,String name, Class model_cls, String col, String cmd) {
    Combo combo = createReadOnlyComboBox(comp);
    DBManager.findByAllToSwtComboBox(combo, model_cls, col);
    combo.addSelectionListener(new CmdSelectionAdapter(this, cmd));
    registControl(name, combo);
    return combo;
  }

  protected Combo addLabelComboByModelOrgRow(String label, String name, Class model_cls,
      String col, String cmd) {
    Composite comp = new Composite(this.sShell, SWT.NO_FOCUS);
    int colspan = 1;
    {
      // 列数を sShell から取得
      GridLayout layout = (GridLayout) this.sShell.getLayout();
      colspan = layout.numColumns;
      comp.setLayout(new GridLayout(2, true));
    }
    GridData compGrid = new GridData();
    compGrid.horizontalAlignment = GridData.FILL;
    // compGrid.verticalAlignment = GridData.FILL;
    compGrid.grabExcessHorizontalSpace = true;
    // compGrid.grabExcessVerticalSpace = true;
    compGrid.horizontalSpan = colspan;

    comp.setLayoutData(compGrid);

    // label を追加
    addLabel(comp, label);

    // combo を格納
    Combo combo = createReadOnlyComboBox(comp);
    DBManager.findByAllToSwtComboBox(combo, model_cls, col);
    combo.addSelectionListener(new CmdSelectionAdapter(this, cmd));
    registControl(name, combo);
    return combo;
  }
  
  
  protected Text addLabelTextOrgRow(String label, String name) {
    Composite comp = new Composite(this.sShell, SWT.NO_FOCUS);
    int colspan = 1;
    {
      // 列数を sShell から取得
      GridLayout layout = (GridLayout) this.sShell.getLayout();
      colspan = layout.numColumns;
      comp.setLayout(new GridLayout(2, true));
    }
    GridData compGrid = new GridData();
    compGrid.horizontalAlignment = GridData.FILL;
    compGrid.grabExcessHorizontalSpace = true;
    compGrid.horizontalSpan = colspan;
    comp.setLayoutData(compGrid);

    // label を追加
    addLabel_right(comp, label);

    // text を格納
    Text text = addTextBox(comp, name);
    registControl(name, text);
    return text;
  }

  protected Combo addComboBox(String name, String[] labels) {
    Combo combo = createReadOnlyComboBox();
    for (int i = 0; i < labels.length; i++) {
      combo.add(labels[i]);
    }
    registControl(name, combo);
    return combo;
  }

  protected Combo addComboBoxEditable(String name, String[] labels, int colspan, int width) {
    Combo combo = createEditableComboBox(colspan, width);
    for (int i = 0; i < labels.length; i++) {
      combo.add(labels[i]);
    }
    registControl(name, combo);
    return combo;
  }

  protected void selectTop(Combo combo) {
    if (combo != null && combo.getItemCount() > 0) {
      combo.select(0);
    }
  }

  /**
   * ラジオボタンを選択する
   * 
   * @param button
   */
  protected void selectItem(Button button) {
    button.setSelection(true);
  }

  protected Table addSimpleTable(Composite comp, String[] cols, int[] wids) {
    return addSimpleTable(comp, cols, wids, 1);
  }

  protected Table addSimpleTable(String[] cols, int[] wids) {
    return addSimpleTable(cols, wids, 1);
  }

  protected Table addSimpleTable(String[] cols, int[] wids, int colspan) {
    return PBControlUtil.createSimpleTable(this.sShell, cols, wids, colspan);
  }

  protected Table addSimpleTable(String[] cols, int[] wids, int colspan, int height) {
    return PBControlUtil.createSimpleTable(this.sShell, cols, wids, colspan, height);
  }

  protected Table addSimpleTable(Composite comp, String[] cols, int[] wids, int colspan) {
    return PBControlUtil.createSimpleTable(comp, cols, wids, colspan);
  }
  protected Table addSimpleTable(Composite comp, String[] cols, int[] wids, int colspan, int height) {
    return PBControlUtil.createSimpleTable(comp, cols, wids, colspan, height);
  }

  protected TableTree addTreeTable(String[] cols, int[] wids, int colspan) {
    return PBControlUtil.createTreeTable(this.sShell, cols, wids, colspan);
  }

  protected Table addMiniTable(String[] cols, int[] wids) {
    Table table = PBControlUtil.createSimpleTable(this.sShell, cols, wids, 1);
    setMinimumHeight(table, 100);
    return table;
  }

  protected void setHorizontalSpan(Control c, int colspan) {
    GridData grid = (GridData) c.getLayoutData();
    grid.horizontalSpan = colspan;
  }

  protected void setVerticalSpan(Control c, int colspan) {
    GridData grid = (GridData) c.getLayoutData();
    grid.verticalSpan = colspan;
  }

  protected void setSpan(Control c, int colspan, int rowspan) {
    PBControlUtil.setSpan(c, colspan, rowspan);
  }

  protected void setWidth(Control c, int width) {
    GridData grid = (GridData) c.getLayoutData();
    grid.widthHint = width;
  }

  protected void setHeight(Control c, int heigth) {
    GridData grid = (GridData) c.getLayoutData();
    grid.heightHint = heigth;
  }

  protected void setMinimumWidth(Control c, int width) {
    GridData grid = (GridData) c.getLayoutData();
    grid.minimumWidth = width;
  }

  protected void setMinimumHeight(Control c, int heigth) {
    GridData grid = (GridData) c.getLayoutData();
    grid.minimumHeight = heigth;
    c.setLayoutData(grid);
  }

  protected void addTableRow(Table table, String[] data) {
    PBControlUtil.addRow(table, data);
  }

  protected void insertTableRowTop(Table table, String[] data) {
    PBControlUtil.insertRowTop(table, data);
  }

  /**
   * @return resultCmd を戻します。
   */
  public String getResultCmd() {
    return resultCmd;
  }

  protected void addLabels(int num, int col_width) {
	    for (int i = 0; i < num; i++) {
	      setWidth(addLabel_left(""), col_width);
	    }
	  }

  protected void addLabels(int num) {
	    for (int i = 0; i < num; i++) {
	    	addLabel();
	    }
	  }

  protected void output1TableToHtml(String template, String title, Table table) {
    PBDocumentBuilder builder = createDocBuilder(template, CHARSET);

    // 保存用ダイアログを開く
    FileDialog saveDialog = new FileDialog(this.sShell, SWT.SAVE);
    saveDialog.setFilterExtensions(new String[] { "*.html" });
    String output_path = saveDialog.open();
    if (output_path == null) {
      return;
    }

    // 1．タイトル
    builder.assign("title", title);

    // 2. メインテーブル
    builder.assign("table", createHtmlTable(table));

    // 6.出力
    builder.output(output_path);

    // 7. 開く
    try {
      Runtime.getRuntime().exec("explorer " + output_path);
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  private static PBDocumentBuilder createDocBuilder(String template, String charset) {
    PBDocumentBuilder builder = PBDocmentBuilderFactory.create(PBDocumentBuilder.HTML, TEMPLATE_DIR
        + template, charset);

    builder.addIncludeFile(CSS);
    assignCommonValue(builder);
    return builder;
  }

  private static String createHtmlTable(Table table) {
    StringBuffer buf = new StringBuffer();

    buf.append("<table><tr>");
    // 1. 列名出力
    for (int i = 0; i < table.getColumnCount(); i++) {
      TableColumn c = table.getColumn(i);
      buf.append("<th>" + c.getText() + "</th>");
    }
    buf.append("</tr>");

    // 2. データ出力
    for (int i = 0; i < table.getItemCount(); i++) {
      TableItem item = table.getItem(i);

      // 全列にデータが無ければ終了
      boolean exist = false;
      for (int c = 0; c < table.getColumnCount() && !exist; c++) {
        exist = item.getText(c).length() > 0;
      }
      if (!exist) {
        break;
      }
      // if (table.getColumnCount() > 0 && item.getText(0).length() == 0) {
      // break;
      // }

      buf.append("<tr>");

      for (int c = 0; c < table.getColumnCount(); c++) {
        buf.append("<td>" + item.getText(c) + "</td>");
      }

      buf.append("</tr>");
    }
    buf.append("</table>");

    return buf.toString();
  }

  /**
   * 共通パラーメータを格納
   * 
   * @param builder
   */
  private static void assignCommonValue(PBDocumentBuilder builder) {
    // date 作成日
    SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd");
    builder.assign("date", df.format(new Date()));

    // footer
    builder
        .assign(
            "footer",
            "<span align=center>published by 2006 Akihiko MONDEN <A href='http://amsoft.minidns.net/'>(AMsoft space)</A></span>");

    // charset ひとまずShift_JIS固定
    builder.assign("charset", CHARSET);
  }

  protected void addShowWindowButton(WindowInfo window_info) {
    Button button = addSimpleButton(window_info.getShowButtonLabel());
    button.addSelectionListener(new MenuSelectAdapter(window_info));
  }

  public class MenuSelectAdapter extends SelectionAdapter {

    private WindowInfo windowInfo;

    public MenuSelectAdapter(WindowInfo windowInfo) {
      super();
      this.windowInfo = windowInfo;
    }

    public void widgetSelected(SelectionEvent e) {
      // ウィンドウクラスによってコンストラクタの引数を変える
      try {
        PBSwt swt;
        Class swt_class = windowInfo.getSwtClass();
        if (PBMasterMgrSwt.class.isAssignableFrom(swt_class)) {
          PBMasterMgrSwt master_swt = (PBMasterMgrSwt) swt_class.getConstructor(
              new Class[] { Class.class }).newInstance(windowInfo.getConstParams());
          // PBMasterMgrSwt master_swt = new PBMasterMgrSwt(swt_class,
          // windowInfo.getShowButtonLabel());
          swt = master_swt;
          swt.setTitle(windowInfo.getShowButtonLabel());
        } else if (PBSwt.class.isAssignableFrom(swt_class)) {
          swt = (PBSwt) swt_class.newInstance();
        } else {
          throw new RuntimeException("invalid swt class on " + windowInfo.getShowButtonLabel());
        }

        swt.show(true);
      } catch (InstantiationException e1) {
        e1.printStackTrace();
      } catch (IllegalAccessException e1) {
        e1.printStackTrace();
      } catch (IllegalArgumentException e1) {
        e1.printStackTrace();
      } catch (SecurityException e1) {
        e1.printStackTrace();
      } catch (InvocationTargetException e1) {
        e1.printStackTrace();
      } catch (NoSuchMethodException e1) {
        e1.printStackTrace();
      }
    }
  }

  /**
   * @param text
   */
  public void setTitle(String text) {
    this.sShell.setText(text);
  }

  /**
   * controlMapの各値をcクラスオブジェクトに格納して返却
   * 
   * @param c
   *          格納先クラス
   * @return 値を格納した cクラスのインスタンス
   */
  protected Object copyControlMapToModel(Class c) {
    Object obj = null;
    try {
      BeanInfo info = Introspector.getBeanInfo(c);
      PropertyDescriptor[] properties = info.getPropertyDescriptors();
      obj = c.newInstance();

      for (int i = 0; i < properties.length; i++) {
        PropertyDescriptor p = properties[i];
        String name = p.getName();
        try {
          if ("class".equals(name) || p.getWriteMethod() == null
              || !this.controlMap.containsKey(name)) {
            continue;
          }

          // 値取得元Control取得
          ControlData cd = ControlDataFactory.createInstance(getControl(name));
          Object val = cd.getBeanValue();

          if (val == null) {
            continue;
          }

          // 値を格納
          if (p.getPropertyType() == Date.class) {
            val = ChkConvSwt.getDate(val.toString());
          } else if (p.getPropertyType() == boolean.class && val instanceof Integer) {
            // boolean かつ Integer 値の場合は boolean に変換
            int intval = ((Integer)val).intValue();
            val = intval == 1 ? "false" : intval == 2 ? "true" : "false";
          }
          BeanUtils.setProperty(obj, name, val);

        } catch (ChkConvSwtRuntimeException e) {
          // エラー発生時のControlをフォーカス
          getControl(name)[0].setFocus();
          throw e;
        }
      }
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (IntrospectionException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    } catch (InstantiationException e) {
      e.printStackTrace();
    }
    return obj;
  }

  protected void copyModelToControlMap(Object obj) {
    Class c = obj.getClass();
    try {
      BeanInfo info = Introspector.getBeanInfo(c);
      PropertyDescriptor[] properties = info.getPropertyDescriptors();

      for (int i = 0; i < properties.length; i++) {
        PropertyDescriptor p = properties[i];
        String name = p.getName();
        try {
          if ("class".equals(name) || p.getWriteMethod() == null
              || !this.controlMap.containsKey(name)) {
            continue;
          }

          // 値取得先Control取得
          ControlData cd = ControlDataFactory.createInstance(getControl(name));
          String val = BeanUtils.getProperty(obj, name);
          if (val == null) {
            continue;
          }
          if (cd instanceof PBComboControlData) {
            setComboValue(c, p, name, cd, val);
          } else {
            cd.setText(val);
          }

        } catch (ChkConvSwtRuntimeException e) {
          // エラー発生時のControlをフォーカス
          getControl(name)[0].setFocus();
          throw e;
        } catch (NoSuchMethodException e) {
          e.printStackTrace();
        }
      }
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (IntrospectionException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    }
  }

  private void setComboValue(Class c, PropertyDescriptor p, String name, ControlData cd, String val) {
    PBComboControlData contdata = (PBComboControlData) cd;
    Combo combo = (Combo) contdata.getControl()[0];

    // コンボの場合
    if (p.getPropertyType() == boolean.class) {
      // boolean の場合は値を判定
      int ind = "1".equals(val) || "true".equals(val.toLowerCase()) ? 1 : 0;
      combo.select(ind);
    } else if (DictUtil.hasFKey(c, name)) {
      // 外部キー有りの場合. そのまま値-1を選択
      combo.select(Integer.parseInt(val) - 1);
    }
  }

  /**
   * clsのnameプロパティのタイプに応じて対応するControlを作成して返す. 汎用画面用
   * 
   * @param cls
   * @param name
   * @return
   */
  protected Control addControlByType(Class cls, String name) {
    Class type = null;
    try {
      type = cls.getDeclaredField(name).getType();
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (NoSuchFieldException e) {
      e.printStackTrace();
    }
    if (type == null) {
      return addTextBox(name);
    } else if (type == boolean.class) {
      // boolean の場合はコンボ
      return addComboBox(name, DictUtil.getBooleanLabels(cls, name));
    } else if (type == int.class && DictUtil.hasFKey(cls, name)) {
      // 外部キーが存在する場合はコンボ
      return addComboBoxByModel(name, DictUtil.getJoinClass(cls, name), DictUtil.getJoinColumn(cls,
          name), null);
    } else {
      return addTextBox(name);
    }
  }

  /**
   * チェックボックスを追加
   * 
   * @param label
   *          ラベル
   */
  protected Button addCheckBox(String label) {
    return addCheckBox(this.sShell, label);
  }

  /**
   * チェックボックスを追加
   * 
   * @param label
   *          ラベル
   */
  protected Button addCheckBox(Composite comp, String label) {
    return PBControlUtil.createCheckBox(comp, label);
  }

  /**
   * テーブルにデータが存在するか確認
   * 
   * @param table
   *          テーブル
   * @param col
   *          列インデックス
   * @param val
   *          値
   * @return true 存在, false 存在しない
   */
  protected boolean tableExist(Table table, int col, String val) {
    if (table == null || col < 0 || val == null) {
      return false;
    }
    for (int i = 0; i < table.getItemCount(); i++) {
      if (val.equals(table.getItem(i).getText(col))) {
        return true;
      }
    }
    return false;
  }
  
	/**
	 * Groupを作成
	 * @param label グループ名ラベル
	 * @param span スパン数
	 * @param equal_width 
	 * @return 生成したGroup
	 */
	protected Group createGroupShadowEtchedOut_FILL(String label, int span, boolean equal_width) {
		return PBControlUtil.createGroupChadowEthedOut_FILL(this.sShell, label, span, equal_width);
	}

	/**
	 * Groupを作成
	 * @param label グループ名ラベル
	 * @param span スパン数
	 * @param equal_width 
	 * @return 生成したGroup
	 */
	protected Group createGroupShadowEtchedOut_BEGIN(String label, int span, boolean equal_width) {
		return PBControlUtil.createGroupChadowEthedOut_BEGIN(this.sShell, label, span, equal_width);
	}

  /**
   * Browserを生成
   * @param colspan 
   * @param group グループ
   * @return ブラウザ
   */
  protected Browser createBrowser(int colspan) {
    return PBControlUtil.createBrowser(this.sShell, colspan);
  }

  /**
   * Browserを生成
   * @param colspan 
   * @param group グループ
   * @return ブラウザ
   */
  protected Browser createBrowser(Composite comp, int colspan) {
    return PBControlUtil.createBrowser(comp, colspan);
  }

  /**
   * @param control
   * @param i
   */
  protected void setShortCutForFocus(Control control, int code) {
    setShortCutForFocus(this.sShell, control, code);
  }
    

  /**
   * @param control
   * @param i
   */
  protected void setShortCutForFocus(Composite comp, Control control, int code) {
    comp.addKeyListener(new FocusKeyListener(control, code));
  }
}
