/**
 * $Id$
 */
package jp.co.powerbeans.kaishuj.logic.parser;

import java.util.ArrayList;
import java.util.List;

import jp.co.powerbeans.common.db.DBManager;
import jp.co.powerbeans.common.string.HtmlUtil;
import jp.co.powerbeans.kaishuj.logic.ClassRemoveMgr;
import jp.co.powerbeans.kaishuj.models.ClassInfo;
import jp.co.powerbeans.kaishuj.models.ConstructorInfo;

/**
 * ConstructorDetailBlockParserImpl
 * 
 * @author amonden
 * @create 2006/12/21
 */
public class ConstructorDetailBlockParserImpl implements BlockParser {

  private ConstructorInfo constInfo;
  private StringBuffer param;
  private StringBuffer expln;
  private List<ConstructorInfo> constInfoList = new ArrayList<ConstructorInfo>();

  /* (非 Javadoc)
   * @see jp.co.powerbeans.kaishuj.logic.parser.BlockParser#parseLine(java.lang.String)
   */
  public void parseLine(String line) {
    String linet = line.trim();
    String linetup = linet.toUpperCase();
    
    if (linetup.startsWith("<A NAME=") && linetup.endsWith("<H3>")) {
      // コンストラクタ定義start
      addConstInfo();
      this.constInfo = new ConstructorInfo();
      this.param = null;
//      this.param = new StringBuffer();
      this.expln = new StringBuffer();
      
    } else if (this.constInfo != null && this.constInfo.getConstructorName() == null &&
          linet.endsWith("</H3>")) {
      this.constInfo.setConstructorName(HtmlUtil.removeTag(linet));
    } else if (linetup.startsWith("<PRE>")) {
      this.param = new StringBuffer();
      this.param.append(HtmlUtil.removeTag(linet));
    }
    
    if (this.param != null) {
      // 引数解析中
      this.param.append(HtmlUtil.removeTag(linet));
    } else if (this.expln != null){
      // 説明解析中
      this.expln.append(HtmlUtil.removeTag(linet));
    }
      
    if (linetup.endsWith("</PRE>")) {
      // 引数定義end
      if (this.constInfo != null && param != null) {
        this.constInfo.setParam(param.toString());
      }
      param = null;
    } else if (linetup.endsWith("<HR>")) {
      // 説明解析end
      if (this.constInfo != null && this.expln != null) {
        this.constInfo.setExplain(this.expln.toString());
      }
      expln = null;
    }
  }

  private void addConstInfo() {
    if (this.constInfo != null) {
      if (this.expln != null) {
        this.constInfo.setExplain(HtmlUtil.removeTag(this.expln.toString()));
        this.expln = null;
      }
      this.constInfoList.add(this.constInfo);
    }
  }

  /* (非 Javadoc)
   * @see jp.co.powerbeans.kaishuj.logic.parser.BlockParser#finish(jp.co.powerbeans.kaishuj.models.ClassInfo)
   */
  public void finish(ClassInfo cls_info) {
    
    // 未追加分を追加
    if (this.constInfo != null) {
      addConstInfo();
    }
    
    // 各リストにidを反映
    ConstructorInfo[] const_infos = (ConstructorInfo[]) this.constInfoList
        .toArray(new ConstructorInfo[this.constInfoList.size()]);
    for (int i = 0; i < const_infos.length; i++) {
      const_infos[i].setClassInfoId(cls_info.getClassInfoId());
    }
    
    // 同情報を削除
    ClassRemoveMgr.removeByClassInfo(ConstructorInfo.class, cls_info);
    
    // 追加
    DBManager.insert(const_infos);
  }

}
