/**
 * $Id$
 */
package jp.co.powerbeans.kaishuj.logic.parser;

import java.util.ArrayList;
import java.util.List;

import jp.co.powerbeans.common.db.DBManager;
import jp.co.powerbeans.common.string.HtmlUtil;
import jp.co.powerbeans.kaishuj.logic.ClassRemoveMgr;
import jp.co.powerbeans.kaishuj.models.ClassInfo;
import jp.co.powerbeans.kaishuj.models.FieldInfo;

/**
 * FieldDetailBlockParserImpl
 * 
 * @author amonden
 * @create 2006/12/21
 */
public class FieldDetailBlockParserImpl implements BlockParser {

  private List<FieldInfo> fieldList = new ArrayList<FieldInfo>();

  private FieldInfo fieldInfo;

  private StringBuffer expln;

  /*
   * (非 Javadoc)
   * 
   * @see jp.co.powerbeans.kaishuj.logic.parser.BlockParser#parseLine(java.lang.String)
   */
  public void parseLine(String line) {

    String linet = line.trim().toUpperCase();
    
    // 新規フィールド
    if (linet.startsWith("<A NAME") && linet.endsWith("<H3>")) {
      this.fieldInfo = new FieldInfo();
    }
    else if (linet.endsWith("</H3>")) {
      // フィールド名取得
      this.fieldInfo.setFieldName(HtmlUtil.removeTag(line));
    }

    // 定義追加
    else if (linet.endsWith("</PRE>")) {
      this.fieldInfo.setFieldDef(HtmlUtil.removeTag(line));
      this.expln = new StringBuffer();
    }
    
    // 説明終了(TODO 関連項目を取得するか)
    else if (linet.startsWith("<DT>")) {
      if (this.expln != null) {
        this.fieldInfo.setExplain(this.expln.toString());
      }
      this.expln = null;
    }
    
    // 説明追加
    else if (this.expln != null) {
      this.expln.append(HtmlUtil.removeTag(line));
    }

    // 終了
    if (linet.trim().startsWith("<HR>")) {
      setFieldInfo();
    }
  }

  private void setFieldInfo() {
    if (this.fieldInfo != null) {
      if (this.expln != null) {
        this.fieldInfo.setExplain(this.expln.toString());
      }
      this.fieldList.add(this.fieldInfo);
      this.fieldInfo = null;
      this.expln = null;
    }
  }

  /*
   * (非 Javadoc)
   * 
   * @see jp.co.powerbeans.kaishuj.logic.parser.BlockParser#finish(jp.co.powerbeans.kaishuj.models.ClassInfo)
   */
  public void finish(ClassInfo cls_info) {
    setFieldInfo();
    
    FieldInfo[] field_infos = this.fieldList.toArray(new FieldInfo[] {});
    for (int i = 0; i < field_infos.length; i++) {
      field_infos[i].setClassInfoId(cls_info.getClassInfoId());
    }

    ClassRemoveMgr.removeByClassInfo(FieldInfo.class, cls_info);

    // 追加
    DBManager.insert(field_infos);
  }

}
