/**
 * $Id$
 */
package jp.co.powerbeans.kaishuj.logic.parser;

import java.util.ArrayList;
import java.util.List;

import jp.co.powerbeans.common.db.DBManager;
import jp.co.powerbeans.common.string.HtmlUtil;
import jp.co.powerbeans.kaishuj.logic.ClassRemoveMgr;
import jp.co.powerbeans.kaishuj.models.ClassInfo;
import jp.co.powerbeans.kaishuj.models.MethodInfo;

/**
 * MethodDetailBlockParserImpl
 * 
 * @author amonden
 * @create 2006/12/21
 */
public class MethodDetailBlockParserImpl implements BlockParser {

  private MethodInfo methodInfo;
  private StringBuffer methodDef;
  private StringBuffer expl;
  private List<MethodInfo> list = new ArrayList<MethodInfo>();

  /* (非 Javadoc)
   * @see jp.co.powerbeans.kaishuj.logic.parser.BlockParser#parseLine(java.lang.String)
   */
  public void parseLine(String line) {
    String linet = line.trim().toUpperCase();
    
    // 新規メソッド
    if (linet.startsWith("<A NAME") && linet.endsWith("<H3>")) {
      this.methodInfo = new MethodInfo();
    }
    // メソッド名取得
    else if (linet.endsWith("</H3>") && this.methodInfo != null) {
      this.methodInfo.setMethodName(HtmlUtil.removeTag(line));
    }
    // 定義開始
    else if (linet.startsWith("<PRE>") && this.methodInfo != null) {
      this.methodDef = new StringBuffer();
    }
    // 説明開始
    else if (linet.startsWith("<DD>") && this.methodInfo != null && this.methodInfo.getExplain() == null) {
      this.expl = new StringBuffer();
    }
    
    // 定義追加
    if (this.methodDef != null) {
      this.methodDef.append(HtmlUtil.removeTagTrim(line));
    } else if (this.expl != null) {
      this.expl.append(HtmlUtil.removeTagTrim(line));
    }
    
    // 定義終了
    if (linet.endsWith("</PRE>") && this.methodInfo != null && this.methodDef != null) {
      this.methodInfo.setMethodDef(this.methodDef.toString());
      this.methodDef = null;
      
    } else if (this.expl != null && this.methodInfo != null && linet.startsWith("<DD>")) {
      this.methodInfo.setExplain(this.expl.toString());
      this.expl = null;
      
    } else if (linet.startsWith("<HR>")) {
      // 1メソッド終了
      addMethodInfo();
    }
  }

  /**
   * メソッド情報をリストに格納
   */
  private void addMethodInfo() {
    if (this.methodInfo != null) {
      this.list.add(this.methodInfo);
      this.methodInfo = new MethodInfo();
    }
  }

  /* (非 Javadoc)
   * @see jp.co.powerbeans.kaishuj.logic.parser.BlockParser#finish(jp.co.powerbeans.kaishuj.models.ClassInfo)
   */
  public void finish(ClassInfo cls_info) {
    addMethodInfo();
    
    MethodInfo[] method_infos = (MethodInfo[]) this.list.toArray(new MethodInfo[]{});
    for (int i = 0; i < method_infos.length; i++) {
      method_infos[i].setClassInfoId(cls_info.getClassInfoId());
    }

    ClassRemoveMgr.removeByClassInfo(MethodInfo.class, cls_info);

    // 追加
    DBManager.insert(method_infos);
  }

}
