package jp.co.powerbeans.kaishuj.swt;

import java.util.Date;

import jp.co.powerbeans.common.chk.ChkConv;
import jp.co.powerbeans.common.db.DBManager;
import jp.co.powerbeans.common.db.DBUtil;
import jp.co.powerbeans.common.swt.ChkConvSwt;
import jp.co.powerbeans.common.swt.MsgSwt;
import jp.co.powerbeans.common.swt.PBSwt;
import jp.co.powerbeans.kaishuj.logic.JavadocRoundLogListener;
import jp.co.powerbeans.kaishuj.logic.JavadocRoundMgr;
import jp.co.powerbeans.kaishuj.models.JavadocUrl;
import jp.co.powerbeans.kaishuj.msg.Msg;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

public class RoundJavaDocSwt extends PBSwt implements JavadocRoundLogListener {

  private static final String TITLE = Msg.getTitleRoundWeb();

  static final int WINDOW_WIDTH = 700;

  static final int WINDOW_HEIGTH = 400;

  static final int BUTTON_WIDTH = WINDOW_WIDTH;

  static final int BUTTON_HEIGTH = 50;

  private static final String[] URL_TABLE_COLS = {
    Msg.getLblId(),
    Msg.getLblModuleName(),
    Msg.getLblURL(),
    Msg.getLblGettedClassNum(),
    Msg.getLblEncode(),
    Msg.getLblLastRoundDt()};

  private static final int[] URL_TABLE_WIDTH = { 50, 100, 200, 120, 100, 130 };

  private static final String[] URL_TABLE_PROPS = { "javadocUrlId",
      "moduleName", "url", "lastRoundObjNum", "encode", "lastRoundDt" };

  private static final int COLSPAN = 4;

  private static final int URL_TABLE_HEIGHT = 150;

  private static final int LOG_HEIGHT = 100;

  //
  private Text logText;

  private Button regCheckBox;

  private Table urlTable;

  private Button roundButton;

  private Group urlTableGrp;

  private Group logTextGrp;

  private Group btnGrp;

  private Button clearLogButton;

  private Text targetClassText;

  public RoundJavaDocSwt() {
    super();
    initialize();
  }

  private void initialize() {
    createSShell(TITLE, WINDOW_WIDTH, WINDOW_HEIGTH, true);
    createItems();
    createMenu();
  }

  /**
   * 
   */
  private void createMenu() {
    // create1LayerMenu(LABELS, CMDS);
  }

  /**
   * 
   */
  private void createItems() {
    // setGridLayout(COLSPAN);

    // 1行目 URL リストテーブル
    this.urlTableGrp = createGroupShadowEtchedOut_FILL( Msg.getLblUrlList(), COLSPAN, true); // FIXME ここから
    this.urlTable = addSimpleTable(this.urlTableGrp, URL_TABLE_COLS,
        URL_TABLE_WIDTH, COLSPAN, URL_TABLE_HEIGHT);
    setCmdToTableOnSelectChange(this.urlTable, "urlTableSelect");
    // setCmdToTableOnSingleClick(this.urlTable, "urlTableSelect");
    // this.urlTable.addSelectionListener()

    // 2行目
    // 対象クラス
    addLabel_right(this.urlTableGrp, Msg.getLblTargetClass() + "(" + Msg.getLblReg() + ")");
    this.targetClassText = addTextBox(this.urlTableGrp, "targetClass");
    
//    巡回ボタン
    this.roundButton = addSimpleButton(this.urlTableGrp, Msg.getLblRoundStart(), "round");
    this.roundButton.setEnabled(false);

    // 6行目 収集ログ
    this.logTextGrp = createGroupShadowEtchedOut_FILL(Msg.getLblLog(), COLSPAN, true);
    this.logText = addTextBoxMultiReadOnly(this.logTextGrp, "logText", 4, LOG_HEIGHT);
    this.clearLogButton = addSimpleButton(this.logTextGrp, Msg.getLblClear(), "clearLog");

    // 初期値格納
    setInitValues();
  }

  /**
   * 
   */
  private void setInitValues() {
    // URLリストテーブル取得
    this.urlTable.removeAll();
    DBUtil.modelsToSwtTable(this.urlTable, DBManager
        .findByAll(JavadocUrl.class), URL_TABLE_PROPS);

//    // ログ初期化
//    this.logText.setText("");
  }

  /**
   * URLテーブル選択
   */
  public void cmdUrlTableSelect() {
    this.roundButton.setEnabled(this.urlTable.getSelectionCount() > 0);
  }

  /**
   * 巡回開始
   */
  public void cmdRound() {
    if (this.urlTable.getSelectionCount() == 0) {
      MsgSwt.warn(Msg.getMsgSelectURL());
      return;
    }
    JavadocUrl[] urls = (JavadocUrl[]) ChkConvSwt
        .getSelectObjects(this.urlTable);
    
      for (int i = 0; i < urls.length; i++) {
        JavadocUrl javadocUrl = urls[i];
        try {

        JavadocRoundMgr mgr = new JavadocRoundMgr(javadocUrl, this,
            this.targetClassText.getText());
        int round_num = mgr.start();

        // 巡回数と巡回日時を更新
        javadocUrl.setLastRoundObjNum(round_num);
        javadocUrl.setLastRoundDt(ChkConv.toStringDT(new Date()));
        DBManager.update(javadocUrl);
      } catch (RuntimeException e) {
        output("Error occured at " + javadocUrl.getUrl() + ", stop download.");
        throw e;
      }
    }    
    
//    setInitValues();
  }

  public void cmdClearLog() {
    this.logText.setText("");
  }

  /*
   * (非 Javadoc)
   * 
   * @see jp.co.powerbeans.kaishuj.logic.JavadocRoundLogListener#output(java.lang.String)
   */
  public void output(String msg) {
    this.logText.append(msg);
    this.logText.append("\n");
    this.logText.update();
  }
}
